@@ -41,11 +41,9 @@ class DatabaseSource( object ):
         session = Session()
         query = session.query( self.domain_model )
         return query
-        
-    def __call__( self, context=None ):
-        query = self.constructQuery( context )
-        results = query.all()
-        
+
+    def results( self, results ):
+        # convert results to a vocabulary
         terms = []
         title_field = self.title_field or self.token_field
         for ob in results:
@@ -55,9 +53,22 @@ class DatabaseSource( object ):
                     token = getattr( ob, self.token_field),
                     title = getattr( ob, title_field) ,
                     ))
-                    
-        return vocabulary.SimpleVocabulary( terms )
+        return vocabulary.SimpleVocabulary( terms )                    
+        
+    def __call__( self, context=None ):
+        query = self.constructQuery( context )
+        values = query.all()
+        return self.results( values )
 
+    def queryTitle( self, query_term ):
+        """
+        search against the title with the given term
+        """
+        values = self.constructQuery().filter(
+                      self.domain_model.c[ self.title_field].like("\%%s\%"%( query_term ) )
+                 ).all()
+        return self.results( values )
+        
 class ObjectSource( DatabaseSource ):
     """
     a vocabulary source, where objects are the values, for suitable for o2m fields.
@@ -67,13 +78,26 @@ class ObjectSource( DatabaseSource ):
         session = Session()
         query = session.query( self.domain_model )
         return query        
-        
+
+    def results( self, values ):
+        terms = [vocabulary.SimpleTerm( value=ob, token=getattr( ob, self.value_field), title=getattr( ob, self.token_field ) ) \
+                 for ob in values ]
+        return terms
+    
     def __call__( self, context=None ):
         query = self.constructQuery( context )
-        results = query.all()
-        terms = [vocabulary.SimpleTerm( value=ob, token=getattr( ob, self.value_field), title=getattr( ob, self.token_field ) ) \
-                 for ob in results ]
-        return vocabulary.SimpleVocabulary( terms )
+        values = query.all()
+        results = self.results( values )
+        return vocabulary.SimpleVocabulary( results )
+
+    def queryTitle( self, query_term ):
+        """
+        search against the title with the given term
+        """
+        values = self.constructQuery().filter(
+                      self.domain_model.c[ self.token_field].like("\%%s\%"%( query_term ) )
+                 ).all()
+        return self.results( values )    
 
 
 class VocabularyTable( object ):
